from typing import Dict, List, Literal, Optional

from pydantic import BaseModel, field_validator

DROP_TYPE_MAPPING = {
    "ONCE": 1,  # 字符串 -> 对应数字
    "NORMAL": 2,
    "SPECIAL": 3,
    "ADDITIONAL": 4,
    "COMPLETE": 8,
}


class displayReward(BaseModel):
    type: str
    id: str
    dropType: Literal[1, 2, 3, 4, 8]  # 最终强制为这几个整数

    @field_validator("dropType", mode="before")
    @classmethod
    def convert_drop_type(cls, value) -> int:
        """将字符串统一转换为预定义的整数"""
        if not isinstance(value, str):
            return value

        value_upper = value.upper()
        if value_upper not in DROP_TYPE_MAPPING:
            raise ValueError(f"未知的掉落类型标识: {value}")
        return DROP_TYPE_MAPPING[value_upper]


class displayDetailReward(BaseModel):
    occPercent: int
    type: str
    id: str
    dropType: Literal[1, 2, 3, 4, 8]  # 最终强制为这几个整数

    @field_validator("dropType", mode="before")
    @classmethod
    def convert_drop_type(cls, value) -> int:
        """将字符串统一转换为预定义的整数"""
        if not isinstance(value, str):
            return value

        value_upper = value.upper()
        if value_upper not in DROP_TYPE_MAPPING:
            raise ValueError(f"未知的掉落类型标识: {value}")
        return DROP_TYPE_MAPPING[value_upper]


class UnlockCondition(BaseModel):
    stageId: str
    completeState: int


# 主模型包含一个 unlockCondition 列表
class StageInfo(BaseModel):
    unlockCondition: List[UnlockCondition]


class StageDropInfo(BaseModel):
    firstPassRewards: Optional[str] = None
    firstCompleteRewards: Optional[str] = None
    passRewards: Optional[str] = None
    completeRewards: Optional[str] = None
    displayRewards: Optional[List[displayReward]] = []
    displayDetailRewards: Optional[List[displayDetailReward]] = []


class ExtraCondition(BaseModel):
    index: int
    template: str
    unlockParam: List[str]


# 定义奖励结构
class Reward(BaseModel):
    id: str
    count: int
    type: str


# 定义 progressInfo
class ProgressInfo(BaseModel):
    progressType: str
    descList: Optional[Dict[str, str]] = None  # 可能为 null 或为空列表


# 定义 extraInfo 中的每个项
class ExtraInfo(BaseModel):
    stageId: str
    rewards: List[Reward]
    progressInfo: ProgressInfo
    imageId: str


class Stage(BaseModel):
    stageType: str
    difficulty: str
    performanceStageFlag: str
    diffGroup: str
    unlockCondition: List[UnlockCondition]
    stageId: str
    levelId: Optional[str] = None
    zoneId: str
    code: str
    name: Optional[str] = None
    description: Optional[str] = None
    hardStagedId: Optional[str] = None  # 修改为 Optional[str]，允许 None
    dangerLevel: Optional[str] = None
    dangerPoint: float
    loadingPicId: str
    canPractice: bool
    canBattleReplay: bool
    apCost: int
    apFailReturn: int
    etItemId: Optional[str] = None
    etCost: int
    etFailReturn: int
    etButtonStyle: Optional[str] = None
    apProtectTimes: int
    diamondOnceDrop: int
    practiceTicketCost: int
    dailyStageDifficulty: Optional[int] = None
    expGain: int
    goldGain: int
    loseExpGain: int
    loseGoldGain: int
    passFavor: int
    completeFavor: int
    slProgress: int
    displayMainItem: Optional[str] = None
    hilightMark: bool  # 修改为 bool
    bossMark: bool  # 修改为 bool
    isPredefined: bool
    isHardPredefined: bool
    isSkillSelectablePredefined: bool
    isStoryOnly: bool

    stageDropInfo: Optional[StageDropInfo] = None  # 修改为模型类型
    canUseCharm: Optional[bool] = False
    canUseTech: Optional[bool] = False
    canUseTrapTool: Optional[bool] = False
    canUseBattlePerformance: Optional[bool] = False
    canUseFirework: Optional[bool] = False
    canContinuousBattle: Optional[bool] = False
    startButtonOverrideId: Optional[str] = None
    isStagePatch: bool
    mainStageId: Optional[str] = None
    extraCondition: Optional[List[ExtraCondition]] = []
    extraInfo: Optional[List[ExtraInfo]] = []
    appearanceStyle: int

    @field_validator("appearanceStyle", mode="before")
    @classmethod
    def convert_appearance_style(cls, value):
        """将字符串或整数统一转换为整数"""
        if isinstance(value, int):
            return value  # 已经是整数，直接返回

        # 处理字符串类型的映射
        mapping = {
            "MAIN_NORMAL": 0,
            "sub": 2,
            # 其他可能的映射...
        }
        if value in mapping:
            return mapping[value]
        else:
            # 未知值处理：抛出错误或设置默认值
            raise ValueError(f"未知的 appearanceStyle 值: {value}")


class stageValidInfo(BaseModel):
    startTs: int
    endTs: int


class Stages(BaseModel):
    stages: Dict[str, Stage]
    stageValidInfo: Dict[str, stageValidInfo]
